/****************************************************************
*
* Copyright (C) 2004 SignalQuest, Inc.
*
*----------------------------------------------------------------
* All software and related documentation is provided "AS IS" and 
* without warranty or support of any kind and SignalQuest 
* expressly disclaims all other warranties, express or implied, 
* including, but not limited to, the implied warranties of 
* merchantability and fitness for a particular purpose. Under no 
* circumstances shall SignalQuest be liable for any 
* incidental, special or consequential damages that result from 
* the use or inability to use the software or related 
* documentation, even if SignalQuest has been advised of 
* the liability.
*
* Unless otherwise stated, software written and copyrighted by 
* SignalQuest is distributed as "freeware". You may use 
* and modify this software without any charge or restriction. 
* You may distribute to others, as long as the original author 
* is acknowledged.
*
*****************************************************************
* 
* Project:  SignalQuest Serial Communications DLL
*
* File:    SQincl.h
*
* History:
*   Version 1.71 (2004-11-10) pmt. Extensive fixes to thread usage.
*   Version 1.5  (2004-08-30)
*
****************************************************************/


// SQIncl.h : main header file for the SQINCL DLL
//

#if !defined(AFX_SQINCL_H__7DD6929F_0C18_4536_AA10_09EDD726B575__INCLUDED_)
#define AFX_SQINCL_H__7DD6929F_0C18_4536_AA10_09EDD726B575__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"		// main symbols


#include "MyWin32Port.h"
#include "RS232Messages.h"



/////////////////////////////////////////////////////////////////////////////
// CSQInclApp
// See SQIncl.cpp for the implementation of this class
//

class CSQInclApp : public CWinApp
{
public:
	CSQInclApp();
	~CSQInclApp();


	// User-defined variables
protected:
	#ifdef _DEBUG
	CMemoryState m_msOld, m_msNew, m_msDif;		// Trace memory leaks
	#endif
	MyWin32Port * m_pPort;						// COM port object
	int m_iPreLoaded;
	int m_iTimeout;
	int m_iGetDataMode;							// Mode for get data routine
	SQPACKET2 m_CurrentPacket;					// Struct for current packet
	int m_iPollingMode;							// Polling to Find Device
	HANDLE m_hPacketReadyEvent;
	char m_szCOM[10];							// Store currently open com port
	int m_iDataSyncMode;						// Data synchronization mode setting
	int m_iAsyncFirstTime;
	int m_iDeletePortRequest;

public:
	int m_iIsClosing;
	int m_iIsOpening;
	HANDLE m_hDeletePortDoneEvent;


	// User-defined functions
protected:
	void CreateDataObjects(void);
	void DestroyDataObjects(void);
	void PeekPause(long liMilliseconds);
	void DeletePort(void);
	void SetPollingModeOn(void);
	void SetPollingModeOff(void);
	void CloseCurrentPort(void);
	void SetDataSyncMode(int iSyncMode);
	void SetDeletePortRequestOn(void);
	void SetDeletePortRequestOff(void);

public:
	LONG Config(
		LONG liMode,
		LONG liBuffSize,
		LONG liRS232Timeout,
		LONG * liRetVal1,
		LONG * liRetVal2);

	LONG OpenPort(
		LPSTR lpszCOMPort,
		LONG liBaudRate,
		LPSTR lpszParity,
		LONG liWordSize,
		LONG liStopBits,
		LONG liXonXoff,
		LONG liRtsCts,
		LONG liDtrDsr);

	LONG CloseCOMPort(LPSTR lpszCOMPort);

	LONG ResetDevice(LPSTR lpszCOMPort);

	LONG ClearLatchedError(LPSTR lpszCOMPort);

	LONG GetData2(
		LPSTR lpszCOMPort,
		FLOAT * pfValue1,
		FLOAT * pfValue2,
		LONG * pliPacketCount,
		LONG * pliPacketError,
		LONG * pliLatchedError,
		LONG * pliBytesInBuffer);

	LONG FindFirstDevice(
		LONG liStartCOM,
		LONG liEndCOM,
		LONG liBaudRate,
		LPSTR lpszParity,
		LONG liWordSize,
		LONG liStopBits,
		LONG liXonXoff,
		LONG liRtsCts,
		LONG liDtrDsr);

	LONG GetErrorString(
		LONG liErrorCode,
		LONG liBuffSize,
		LPTSTR lptszBuffer);

	LONG GetCTSStatus(
		LPSTR lpszCOMPort,
		LONG * pliCTSStatus);

	LONG GetDSRStatus(
		LPSTR lpszCOMPort,
		LONG * pliDSRStatus);

	LONG SetRTSStatus(
		LPSTR lpszCOMPort,
		LONG liRTSStatus);

	LONG SetDTRStatus(
		LPSTR lpszCOMPort,
		LONG liDTRStatus);

	LONG GetPreloadedState(void);		// If DLL has been loaded
	int GetPollingMode(void);			// If find next device (port polling)
	int GetDataSyncMode(void);			// Return current get data (sync) mode
	int GetDeletePortRequest(void);


//m_pPort->GetDSRStatus();
//m_pPort->GetCTSStatus();
//m_pPort->GetDSRChangeEvent();
//m_pPort->GetCTSChangeEvent();


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSQInclApp)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CSQInclApp)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_SQIncl_H__7DD6929F_0C18_4536_AA10_09EDD726B575__INCLUDED_)
