/****************************************************************
*
* Copyright (C) 2004 SignalQuest, Inc.
*
*----------------------------------------------------------------
* All software and related documentation is provided "AS IS" and 
* without warranty or support of any kind and SignalQuest 
* expressly disclaims all other warranties, express or implied, 
* including, but not limited to, the implied warranties of 
* merchantability and fitness for a particular purpose. Under no 
* circumstances shall SignalQuest be liable for any 
* incidental, special or consequential damages that result from 
* the use or inability to use the software or related 
* documentation, even if SignalQuest has been advised of 
* the liability.
*
* Unless otherwise stated, software written and copyrighted by 
* SignalQuest is distributed as "freeware". You may use 
* and modify this software without any charge or restriction. 
* You may distribute to others, as long as the original author 
* is acknowledged.
*
*****************************************************************
* 
* Project:  SignalQuest Serial Communications DLL Demo Program in C#
*
* File:    Form1.cs
*
* History:
*   Version 1.71 (2004-11-14)
*
****************************************************************/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;
using System.Text;
using System.IO;


namespace Test1
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class TestSQIncl : System.Windows.Forms.Form
	{
		const string m_DLL_NAME = "SQIncl.dll";

		// User-defined variables (properties)
		private int m_iCOMPortIsOpen;
		private int m_iLoopTime;
		private int m_iDataSyncMode;
		private int m_iPacketCount;
		private int m_iDroppedPackets;
		private int m_iLatchedError;
		private int m_iDuplicatePackets;
		private int m_iGetDataBusy;
		private int m_iStatusLineCount;
		private uint m_uiDataReadyEvent;
		private int MODE_INIT_DLL;
		private int MODE_STOP;
		private int MODE_SYNC;
		private int MODE_ASYNC_NO_EVENT;
		private int MODE_ASYNC_EVENT;

		// Form editor-generated component objects
		private System.Windows.Forms.Label LabelCom;
		private System.Windows.Forms.Label LabelTitle;
		private System.Windows.Forms.Label LabelLoopTimer;
		private System.Windows.Forms.Label LabelPacketCount;
		private System.Windows.Forms.Label LabelDroppedPackets;
		private System.Windows.Forms.Label LabelLatchedError;
		private System.Windows.Forms.Label LabelBaudEtc;
		private System.Windows.Forms.Button ButtonOpenCOM;
		private System.Windows.Forms.Button ButtonCloseCOM;
		private System.Windows.Forms.Button ButtonGetData;
		private System.Windows.Forms.Button ButtonBeginStream;
		private System.Windows.Forms.Button ButtonEndStream;
		private System.Windows.Forms.Button ButtonClearLatchedError;
		private System.Windows.Forms.Button ButtonSendReset;
		private System.Windows.Forms.NumericUpDown EditCOMPort;
		private System.Windows.Forms.TextBox EditLoopTimer;
		private System.Windows.Forms.TextBox EditPacketCount;
		private System.Windows.Forms.TextBox EditDroppedPackets;
		private System.Windows.Forms.TextBox EditLatchedError;
		private System.Windows.Forms.Timer Timer1;
		private System.Windows.Forms.Button ButtonFindDevice;
		private System.Windows.Forms.Button ButtonDone;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.GroupBox groupBox3;
		private System.Windows.Forms.GroupBox groupBox4;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.TextBox EditDuplicatePackets;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.ComboBox cbDataSyncMode;
		private System.Windows.Forms.ListBox lbStatusText;
		private System.Windows.Forms.TextBox EditDataPoints;
		private System.Windows.Forms.Button ButtonLinearCodeTest;
		private System.ComponentModel.IContainer components;


		// Class constructor
		public TestSQIncl()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();


			// Variable initializations
			m_iLoopTime = 10;
			m_iCOMPortIsOpen = 0;
			m_iDataSyncMode = MODE_INIT_DLL;
			m_iPacketCount = 0;
			m_iDroppedPackets = 0;
			m_iLatchedError = 0;
			m_iDuplicatePackets = 0;
			m_iGetDataBusy = 0;
			m_iStatusLineCount = 0;
			m_uiDataReadyEvent = 0;

			MODE_INIT_DLL = -1;
			MODE_STOP = 0;
			MODE_SYNC = 1;
			MODE_ASYNC_NO_EVENT = 2;
			MODE_ASYNC_EVENT = 3;


			// Initialize DLL
			int RetVal1 = 0;
			int RetVal2 = 0;

			SQConfig(MODE_INIT_DLL, 0, 75, ref RetVal1, ref RetVal2);

			cbDataSyncMode.SelectedIndex = 0;
		}


		// Class destructor
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		// User-defined functions (methods)

		// DLL Import declarations
		[DllImport(m_DLL_NAME)]
		public static extern int SQConfig(
			int liMode,
			int liBuffSize,
			int liTimeout,
			ref int liRetVal1,
			ref int liRetVal2);

		[DllImport(m_DLL_NAME)]
		public static extern int SQOpenPort(
			int liCOMPort,
			int liBaudRate,
			string szParity,
			int liWordSize,
			int liStopBits,
			int liXonXoff,
			int liRtsCts,
			int liDtrDsr);

		[DllImport(m_DLL_NAME)]
		public static extern int SQClosePort(int liCOMPort);

		[DllImport(m_DLL_NAME)]
		public static extern int SQResetDevice(int liCOMPort);

		[DllImport(m_DLL_NAME)]
		public static extern int SQClearLatchedError(int liCOMPort);

		[DllImport(m_DLL_NAME)]
		public static extern int SQGetData2(
			int liCOMPort,
			ref float Pitch,
			ref float Roll,
			ref int PacketCount,
			ref int CurrentError,
			ref int LatchedError);

		[DllImport(m_DLL_NAME)]
		public static extern int SQFindFirstDevice(
			int liCOMStart,
			int liCOMEnd,
			int liBaudRate,
			string szParity,
			int liWordSize,
			int liStopBits,
			int liXonXoff,
			int liRtsCts,
			int liDtrDsr);

		[DllImport(m_DLL_NAME)]
		public static extern int SQGetErrorString(
			int liErrorCode,
			int liBuffSize,
			StringBuilder sbBuffer);

		[DllImport("kernel32",EntryPoint="WaitForSingleObject",
			 SetLastError=true,CharSet=CharSet.Unicode)]
		private static extern uint NT_WaitForSingleObject(
			uint hHandle,
			uint dwMilliseconds);


		// Get data routine
		private int GetData(int iOptCom)
		{
			// Get Inclinometer Data
			float fPitch = 0.0F;
			float fRoll = 0.0F;
			int iPacketCount = 0;
			int iCurrentError = 0;
			int iLatchedError = 0;
			int iErrorCode = 0;
			int iUpdatePackets = 1;
			string szPitchRoll;
			int iComPort;

			if(iOptCom > 0)
			{
				iComPort = iOptCom;
			}
			else
			{
				iComPort = Convert.ToInt32(EditCOMPort.Value);
			}


			if(m_iDataSyncMode == MODE_ASYNC_EVENT)
			{
				NT_WaitForSingleObject(m_uiDataReadyEvent, 2000);
			}


			// Acquire one data point
			iErrorCode = SQGetData2(
				iComPort,
				ref fPitch,
				ref fRoll,
				ref iPacketCount,
				ref iCurrentError,
				ref iLatchedError);


			// Check liResult for error
			if(iErrorCode != 0)
			{
				DisplayErrorText(iErrorCode);

				// If not checksum error
				if(iErrorCode != -110)
				{
					iUpdatePackets = 0;
				}
			}
			else
			{
				// No error, update pitch and roll
				szPitchRoll = "Pitch:  " + Convert.ToString(fPitch) + "    Roll:  " + Convert.ToString(fRoll);
				SetStatusText(szPitchRoll);
			}

			if(iUpdatePackets == 1)
			{
				// Packet count difference from last read to this read
				int iPacketDiff = iPacketCount - m_iPacketCount;

				// Normal packet counter = increase by one
				if(iPacketDiff == 1)
				{
				}

				// If at least one packet was dropped
				if(iPacketDiff > 1)
				{
					if(!((m_iDataSyncMode == MODE_SYNC) && (m_iPacketCount == 0)))
					{
						// Increment dropped packet counter
						m_iDroppedPackets += iPacketDiff;
					}
				}

				// If packet diff is negative
				if(iPacketDiff < 0)
				{
					// Unsigned short rollover case
					iPacketDiff = iPacketDiff + 65536;
				}

				// If same packet read twice in a row or more
				if(iPacketDiff == 0)
				{
					// Buffer has not been updated yet
					m_iDuplicatePackets++;

					// If several duplicate packets are read in a row
					if(m_iDuplicatePackets >= 3)
					{
						// Update status window
						iErrorCode = -75;
					}
				}
				else
				{
					// Zero out duplicate packet read counter
					m_iDuplicatePackets = 0;
				}

				// Update stored packet counter and edit box
				m_iPacketCount = iPacketCount;
				EditPacketCount.Text = Convert.ToString(m_iPacketCount);
				EditDroppedPackets.Text = Convert.ToString(m_iDroppedPackets);
				EditDuplicatePackets.Text = Convert.ToString(m_iDuplicatePackets);
			}


			return iErrorCode;
		}

		private void SetStatusText(string szString)
		{
			if(m_iStatusLineCount > 30)
			{
				lbStatusText.Items.RemoveAt(0);
			}
			else
			{
				m_iStatusLineCount++;
			}


			lbStatusText.Items.Add(szString);
			lbStatusText.Invalidate();
			Update();
		}

		private void DisplayErrorText(int liErrorCode)
		{
			StringBuilder szBuffer1 = new StringBuilder("", 80);
			string szBuffer2 = "";


			int iResult = SQGetErrorString(liErrorCode, 80, szBuffer1);


			if(iResult == 0)
			{
				szBuffer2 = "Error:  " + Convert.ToString(liErrorCode) + "    " + szBuffer1;
			}
			else
			{
				szBuffer2 = "Error:  " + Convert.ToString(liErrorCode);
			}


			SetStatusText(szBuffer2);
		}

		private void DataModeSwitch()
		{
			int liVal1 = 0;
			int liVal2 = 0;


			SQConfig(MODE_STOP, 0, 75, ref liVal1, ref liVal2);


			int iCurSel = cbDataSyncMode.SelectedIndex;


			if(iCurSel == 0)
			{
				m_iDataSyncMode = MODE_SYNC;
				SQConfig(MODE_SYNC, 0, 75, ref liVal1, ref liVal2);
			}
			else if(iCurSel == 1)
			{
				m_iDataSyncMode = MODE_ASYNC_NO_EVENT;
				SQConfig(MODE_ASYNC_NO_EVENT, 0, 75, ref liVal1, ref liVal2);
			}
			else if(iCurSel == 2)
			{
				m_iDataSyncMode = MODE_ASYNC_EVENT;
				SQConfig(MODE_ASYNC_EVENT, 0, 75, ref liVal1, ref liVal2);
				m_uiDataReadyEvent = (uint)liVal1;
			}
		}


		// Auto-generated functions (event handlers)
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(TestSQIncl));
			this.ButtonOpenCOM = new System.Windows.Forms.Button();
			this.ButtonCloseCOM = new System.Windows.Forms.Button();
			this.ButtonGetData = new System.Windows.Forms.Button();
			this.LabelCom = new System.Windows.Forms.Label();
			this.LabelTitle = new System.Windows.Forms.Label();
			this.EditCOMPort = new System.Windows.Forms.NumericUpDown();
			this.ButtonBeginStream = new System.Windows.Forms.Button();
			this.ButtonEndStream = new System.Windows.Forms.Button();
			this.Timer1 = new System.Windows.Forms.Timer(this.components);
			this.ButtonClearLatchedError = new System.Windows.Forms.Button();
			this.EditLoopTimer = new System.Windows.Forms.TextBox();
			this.EditPacketCount = new System.Windows.Forms.TextBox();
			this.EditDroppedPackets = new System.Windows.Forms.TextBox();
			this.EditLatchedError = new System.Windows.Forms.TextBox();
			this.LabelLoopTimer = new System.Windows.Forms.Label();
			this.LabelPacketCount = new System.Windows.Forms.Label();
			this.LabelDroppedPackets = new System.Windows.Forms.Label();
			this.LabelLatchedError = new System.Windows.Forms.Label();
			this.ButtonSendReset = new System.Windows.Forms.Button();
			this.LabelBaudEtc = new System.Windows.Forms.Label();
			this.ButtonFindDevice = new System.Windows.Forms.Button();
			this.ButtonDone = new System.Windows.Forms.Button();
			this.ButtonLinearCodeTest = new System.Windows.Forms.Button();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.cbDataSyncMode = new System.Windows.Forms.ComboBox();
			this.label1 = new System.Windows.Forms.Label();
			this.EditDuplicatePackets = new System.Windows.Forms.TextBox();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.EditDataPoints = new System.Windows.Forms.TextBox();
			this.label3 = new System.Windows.Forms.Label();
			this.groupBox4 = new System.Windows.Forms.GroupBox();
			this.lbStatusText = new System.Windows.Forms.ListBox();
			this.label2 = new System.Windows.Forms.Label();
			((System.ComponentModel.ISupportInitialize)(this.EditCOMPort)).BeginInit();
			this.groupBox1.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.groupBox3.SuspendLayout();
			this.groupBox4.SuspendLayout();
			this.SuspendLayout();
			// 
			// ButtonOpenCOM
			// 
			this.ButtonOpenCOM.Location = new System.Drawing.Point(16, 104);
			this.ButtonOpenCOM.Name = "ButtonOpenCOM";
			this.ButtonOpenCOM.Size = new System.Drawing.Size(128, 24);
			this.ButtonOpenCOM.TabIndex = 0;
			this.ButtonOpenCOM.Text = "Open COM Port";
			this.ButtonOpenCOM.Click += new System.EventHandler(this.ButtonOpenCOM_Click);
			// 
			// ButtonCloseCOM
			// 
			this.ButtonCloseCOM.Location = new System.Drawing.Point(16, 136);
			this.ButtonCloseCOM.Name = "ButtonCloseCOM";
			this.ButtonCloseCOM.Size = new System.Drawing.Size(128, 24);
			this.ButtonCloseCOM.TabIndex = 1;
			this.ButtonCloseCOM.Text = "Close COM Port";
			this.ButtonCloseCOM.Click += new System.EventHandler(this.ButtonCloseCOM_Click);
			// 
			// ButtonGetData
			// 
			this.ButtonGetData.Location = new System.Drawing.Point(16, 72);
			this.ButtonGetData.Name = "ButtonGetData";
			this.ButtonGetData.Size = new System.Drawing.Size(128, 24);
			this.ButtonGetData.TabIndex = 2;
			this.ButtonGetData.Text = "Get Single Packet";
			this.ButtonGetData.Click += new System.EventHandler(this.ButtonGetData_Click);
			// 
			// LabelCom
			// 
			this.LabelCom.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.LabelCom.Location = new System.Drawing.Point(160, 104);
			this.LabelCom.Name = "LabelCom";
			this.LabelCom.Size = new System.Drawing.Size(72, 16);
			this.LabelCom.TabIndex = 4;
			this.LabelCom.Text = "COM Port";
			// 
			// LabelTitle
			// 
			this.LabelTitle.Font = new System.Drawing.Font("Times New Roman", 21.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.LabelTitle.ImageAlign = System.Drawing.ContentAlignment.MiddleLeft;
			this.LabelTitle.Location = new System.Drawing.Point(16, 8);
			this.LabelTitle.Name = "LabelTitle";
			this.LabelTitle.Size = new System.Drawing.Size(688, 32);
			this.LabelTitle.TabIndex = 5;
			this.LabelTitle.Text = "SignalQuest Windows Serial Communications API VC#";
			this.LabelTitle.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// EditCOMPort
			// 
			this.EditCOMPort.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.EditCOMPort.Location = new System.Drawing.Point(272, 104);
			this.EditCOMPort.Maximum = new System.Decimal(new int[] {
																		256,
																		0,
																		0,
																		0});
			this.EditCOMPort.Minimum = new System.Decimal(new int[] {
																		1,
																		0,
																		0,
																		0});
			this.EditCOMPort.Name = "EditCOMPort";
			this.EditCOMPort.Size = new System.Drawing.Size(56, 20);
			this.EditCOMPort.TabIndex = 10;
			this.EditCOMPort.Value = new System.Decimal(new int[] {
																	  1,
																	  0,
																	  0,
																	  0});
			// 
			// ButtonBeginStream
			// 
			this.ButtonBeginStream.Location = new System.Drawing.Point(16, 104);
			this.ButtonBeginStream.Name = "ButtonBeginStream";
			this.ButtonBeginStream.Size = new System.Drawing.Size(128, 24);
			this.ButtonBeginStream.TabIndex = 11;
			this.ButtonBeginStream.Text = "Start Packet Loop";
			this.ButtonBeginStream.Click += new System.EventHandler(this.ButtonStartPacketLoop_Click);
			// 
			// ButtonEndStream
			// 
			this.ButtonEndStream.Location = new System.Drawing.Point(16, 136);
			this.ButtonEndStream.Name = "ButtonEndStream";
			this.ButtonEndStream.Size = new System.Drawing.Size(128, 24);
			this.ButtonEndStream.TabIndex = 12;
			this.ButtonEndStream.Text = "Stop Packet Loop";
			this.ButtonEndStream.Click += new System.EventHandler(this.ButtonEndPacketLoop_Click);
			// 
			// Timer1
			// 
			this.Timer1.Tick += new System.EventHandler(this.Timer1_Tick);
			// 
			// ButtonClearLatchedError
			// 
			this.ButtonClearLatchedError.Location = new System.Drawing.Point(16, 200);
			this.ButtonClearLatchedError.Name = "ButtonClearLatchedError";
			this.ButtonClearLatchedError.Size = new System.Drawing.Size(128, 24);
			this.ButtonClearLatchedError.TabIndex = 16;
			this.ButtonClearLatchedError.Text = "Clear Latched Error";
			this.ButtonClearLatchedError.Click += new System.EventHandler(this.ButtonClearLatchedError_Click);
			// 
			// EditLoopTimer
			// 
			this.EditLoopTimer.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.EditLoopTimer.Location = new System.Drawing.Point(272, 72);
			this.EditLoopTimer.Name = "EditLoopTimer";
			this.EditLoopTimer.Size = new System.Drawing.Size(56, 20);
			this.EditLoopTimer.TabIndex = 18;
			this.EditLoopTimer.Text = "15";
			// 
			// EditPacketCount
			// 
			this.EditPacketCount.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.EditPacketCount.Location = new System.Drawing.Point(272, 104);
			this.EditPacketCount.Name = "EditPacketCount";
			this.EditPacketCount.ReadOnly = true;
			this.EditPacketCount.Size = new System.Drawing.Size(56, 20);
			this.EditPacketCount.TabIndex = 19;
			this.EditPacketCount.Text = "0";
			// 
			// EditDroppedPackets
			// 
			this.EditDroppedPackets.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.EditDroppedPackets.Location = new System.Drawing.Point(272, 136);
			this.EditDroppedPackets.Name = "EditDroppedPackets";
			this.EditDroppedPackets.ReadOnly = true;
			this.EditDroppedPackets.Size = new System.Drawing.Size(56, 20);
			this.EditDroppedPackets.TabIndex = 20;
			this.EditDroppedPackets.Text = "0";
			// 
			// EditLatchedError
			// 
			this.EditLatchedError.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.EditLatchedError.Location = new System.Drawing.Point(272, 200);
			this.EditLatchedError.Name = "EditLatchedError";
			this.EditLatchedError.ReadOnly = true;
			this.EditLatchedError.Size = new System.Drawing.Size(56, 20);
			this.EditLatchedError.TabIndex = 21;
			this.EditLatchedError.Text = "0";
			// 
			// LabelLoopTimer
			// 
			this.LabelLoopTimer.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.LabelLoopTimer.Location = new System.Drawing.Point(160, 72);
			this.LabelLoopTimer.Name = "LabelLoopTimer";
			this.LabelLoopTimer.Size = new System.Drawing.Size(96, 16);
			this.LabelLoopTimer.TabIndex = 22;
			this.LabelLoopTimer.Text = "Loop Timer (ms)";
			// 
			// LabelPacketCount
			// 
			this.LabelPacketCount.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.LabelPacketCount.Location = new System.Drawing.Point(160, 104);
			this.LabelPacketCount.Name = "LabelPacketCount";
			this.LabelPacketCount.Size = new System.Drawing.Size(80, 16);
			this.LabelPacketCount.TabIndex = 23;
			this.LabelPacketCount.Text = "Packet Count";
			// 
			// LabelDroppedPackets
			// 
			this.LabelDroppedPackets.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.LabelDroppedPackets.Location = new System.Drawing.Point(160, 136);
			this.LabelDroppedPackets.Name = "LabelDroppedPackets";
			this.LabelDroppedPackets.Size = new System.Drawing.Size(96, 16);
			this.LabelDroppedPackets.TabIndex = 24;
			this.LabelDroppedPackets.Text = "Dropped Packets";
			// 
			// LabelLatchedError
			// 
			this.LabelLatchedError.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.LabelLatchedError.Location = new System.Drawing.Point(160, 200);
			this.LabelLatchedError.Name = "LabelLatchedError";
			this.LabelLatchedError.Size = new System.Drawing.Size(88, 16);
			this.LabelLatchedError.TabIndex = 25;
			this.LabelLatchedError.Text = "Latched Error";
			// 
			// ButtonSendReset
			// 
			this.ButtonSendReset.Location = new System.Drawing.Point(16, 168);
			this.ButtonSendReset.Name = "ButtonSendReset";
			this.ButtonSendReset.Size = new System.Drawing.Size(128, 24);
			this.ButtonSendReset.TabIndex = 27;
			this.ButtonSendReset.Text = "Send Reset";
			this.ButtonSendReset.Click += new System.EventHandler(this.ButtonSendReset_Click);
			// 
			// LabelBaudEtc
			// 
			this.LabelBaudEtc.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.LabelBaudEtc.Location = new System.Drawing.Point(48, 72);
			this.LabelBaudEtc.Name = "LabelBaudEtc";
			this.LabelBaudEtc.Size = new System.Drawing.Size(248, 16);
			this.LabelBaudEtc.TabIndex = 28;
			this.LabelBaudEtc.Text = "115,200 Kbaud, 8 bits, 1 stop bit, no flow control";
			// 
			// ButtonFindDevice
			// 
			this.ButtonFindDevice.Location = new System.Drawing.Point(64, 32);
			this.ButtonFindDevice.Name = "ButtonFindDevice";
			this.ButtonFindDevice.Size = new System.Drawing.Size(216, 24);
			this.ButtonFindDevice.TabIndex = 29;
			this.ButtonFindDevice.Text = "Find First Device (COM1 - COM20)";
			this.ButtonFindDevice.Click += new System.EventHandler(this.ButtonFindFirstDevice_Click);
			// 
			// ButtonDone
			// 
			this.ButtonDone.Location = new System.Drawing.Point(120, 464);
			this.ButtonDone.Name = "ButtonDone";
			this.ButtonDone.Size = new System.Drawing.Size(104, 24);
			this.ButtonDone.TabIndex = 30;
			this.ButtonDone.Text = "Done";
			this.ButtonDone.Click += new System.EventHandler(this.ButtonDone_Click);
			// 
			// ButtonLinearCodeTest
			// 
			this.ButtonLinearCodeTest.Location = new System.Drawing.Point(16, 32);
			this.ButtonLinearCodeTest.Name = "ButtonLinearCodeTest";
			this.ButtonLinearCodeTest.Size = new System.Drawing.Size(128, 24);
			this.ButtonLinearCodeTest.TabIndex = 32;
			this.ButtonLinearCodeTest.Text = "Linear Code Test";
			this.ButtonLinearCodeTest.Click += new System.EventHandler(this.ButtonLinearCodeTest_Click);
			// 
			// groupBox1
			// 
			this.groupBox1.Controls.Add(this.ButtonFindDevice);
			this.groupBox1.Controls.Add(this.ButtonCloseCOM);
			this.groupBox1.Controls.Add(this.ButtonOpenCOM);
			this.groupBox1.Controls.Add(this.EditCOMPort);
			this.groupBox1.Controls.Add(this.LabelCom);
			this.groupBox1.Controls.Add(this.LabelBaudEtc);
			this.groupBox1.Location = new System.Drawing.Point(16, 48);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(344, 176);
			this.groupBox1.TabIndex = 35;
			this.groupBox1.TabStop = false;
			this.groupBox1.Text = "COM Port Settings";
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.Add(this.cbDataSyncMode);
			this.groupBox2.Controls.Add(this.LabelLoopTimer);
			this.groupBox2.Controls.Add(this.EditLoopTimer);
			this.groupBox2.Controls.Add(this.EditPacketCount);
			this.groupBox2.Controls.Add(this.EditDroppedPackets);
			this.groupBox2.Controls.Add(this.EditLatchedError);
			this.groupBox2.Controls.Add(this.LabelPacketCount);
			this.groupBox2.Controls.Add(this.LabelDroppedPackets);
			this.groupBox2.Controls.Add(this.LabelLatchedError);
			this.groupBox2.Controls.Add(this.ButtonClearLatchedError);
			this.groupBox2.Controls.Add(this.ButtonSendReset);
			this.groupBox2.Controls.Add(this.ButtonEndStream);
			this.groupBox2.Controls.Add(this.ButtonBeginStream);
			this.groupBox2.Controls.Add(this.ButtonGetData);
			this.groupBox2.Controls.Add(this.label1);
			this.groupBox2.Controls.Add(this.EditDuplicatePackets);
			this.groupBox2.Location = new System.Drawing.Point(16, 232);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(344, 240);
			this.groupBox2.TabIndex = 36;
			this.groupBox2.TabStop = false;
			this.groupBox2.Text = "Data Acquisition Control";
			// 
			// cbDataSyncMode
			// 
			this.cbDataSyncMode.Items.AddRange(new object[] {
																"Synchronous Mode",
																"Asynchronous Mode",
																"Async Event-Driven Mode"});
			this.cbDataSyncMode.Location = new System.Drawing.Point(120, 32);
			this.cbDataSyncMode.Name = "cbDataSyncMode";
			this.cbDataSyncMode.Size = new System.Drawing.Size(208, 21);
			this.cbDataSyncMode.TabIndex = 41;
			// 
			// label1
			// 
			this.label1.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label1.Location = new System.Drawing.Point(160, 168);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(96, 16);
			this.label1.TabIndex = 39;
			this.label1.Text = "Duplicate Packets";
			// 
			// EditDuplicatePackets
			// 
			this.EditDuplicatePackets.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.EditDuplicatePackets.Location = new System.Drawing.Point(272, 168);
			this.EditDuplicatePackets.Name = "EditDuplicatePackets";
			this.EditDuplicatePackets.ReadOnly = true;
			this.EditDuplicatePackets.Size = new System.Drawing.Size(56, 20);
			this.EditDuplicatePackets.TabIndex = 40;
			this.EditDuplicatePackets.Text = "0";
			// 
			// groupBox3
			// 
			this.groupBox3.Controls.Add(this.EditDataPoints);
			this.groupBox3.Controls.Add(this.label3);
			this.groupBox3.Controls.Add(this.ButtonLinearCodeTest);
			this.groupBox3.Location = new System.Drawing.Point(16, 480);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(344, 72);
			this.groupBox3.TabIndex = 37;
			this.groupBox3.TabStop = false;
			this.groupBox3.Text = "Special Functions";
			// 
			// EditDataPoints
			// 
			this.EditDataPoints.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.EditDataPoints.Location = new System.Drawing.Point(272, 32);
			this.EditDataPoints.Name = "EditDataPoints";
			this.EditDataPoints.Size = new System.Drawing.Size(56, 20);
			this.EditDataPoints.TabIndex = 34;
			this.EditDataPoints.Text = "25";
			// 
			// label3
			// 
			this.label3.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label3.Location = new System.Drawing.Point(160, 32);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(88, 16);
			this.label3.TabIndex = 33;
			this.label3.Text = "Data Points";
			// 
			// groupBox4
			// 
			this.groupBox4.Controls.Add(this.lbStatusText);
			this.groupBox4.Controls.Add(this.ButtonDone);
			this.groupBox4.Location = new System.Drawing.Point(376, 48);
			this.groupBox4.Name = "groupBox4";
			this.groupBox4.Size = new System.Drawing.Size(344, 504);
			this.groupBox4.TabIndex = 38;
			this.groupBox4.TabStop = false;
			this.groupBox4.Text = "Message Console";
			// 
			// lbStatusText
			// 
			this.lbStatusText.Items.AddRange(new object[] {
															  "SignalQuest Windows Serial Communications API"});
			this.lbStatusText.Location = new System.Drawing.Point(16, 24);
			this.lbStatusText.Name = "lbStatusText";
			this.lbStatusText.Size = new System.Drawing.Size(312, 420);
			this.lbStatusText.TabIndex = 40;
			// 
			// label2
			// 
			this.label2.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.label2.Location = new System.Drawing.Point(32, 264);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(88, 16);
			this.label2.TabIndex = 39;
			this.label2.Text = "Mode Control";
			// 
			// TestSQIncl
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(736, 573);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.groupBox4);
			this.Controls.Add(this.LabelTitle);
			this.Controls.Add(this.groupBox1);
			this.Controls.Add(this.groupBox2);
			this.Controls.Add(this.groupBox3);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "TestSQIncl";
			this.Text = "TestSQIncl";
			((System.ComponentModel.ISupportInitialize)(this.EditCOMPort)).EndInit();
			this.groupBox1.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			this.groupBox4.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			// Look for DLL
			string[] dirs = Directory.GetFiles(@Directory.GetCurrentDirectory(), m_DLL_NAME);


			// If file found
			if(dirs.Length > 0)
			{
				// Start application object
				Application.Run(new TestSQIncl());
			}
			else
			{
				// Exit application
				MessageBox.Show(
					"Error:  SQIncl.dll not found.",
					"TestSQIncl", 
					MessageBoxButtons.OK,
					MessageBoxIcon.Exclamation);
			}
		}

		private void ButtonOpenCOM_Click(object sender, System.EventArgs e)
		{
			// Open COM Port
			int iComPort = Convert.ToInt32(EditCOMPort.Value);


			SetStatusText("Attempting to open COM" + EditCOMPort.Value);
			SetStatusText("This may take some time ...");


			int result = SQOpenPort(
				iComPort,
				115200,
				"N",
				8,
				1,
				0,
				0,
				0);

			if(result != 0)
			{
				DisplayErrorText(result);
			}
			else
			{
				m_iCOMPortIsOpen = 1;

				SetStatusText("Opened COM"+EditCOMPort.Value);
				EditCOMPort.Enabled = false;
			}
		}

		private void ButtonCloseCOM_Click(object sender, System.EventArgs e)
		{
			// Close COM Port
			int iComPort = Convert.ToInt32(EditCOMPort.Value);

			if(Timer1.Enabled == true)
			{
				SetStatusText("Must Stop Timer First");
			}
			else
			{
				SetStatusText("Attempting to close COM" + EditCOMPort.Value);
				SetStatusText("This may take some time ...");


				int result = SQClosePort(iComPort);


				if(result != 0)
				{
					DisplayErrorText(result);
				}
				else
				{
					m_iCOMPortIsOpen = 0;
					
					m_iDataSyncMode = MODE_SYNC;
					
					SetStatusText("Closed COM"+EditCOMPort.Value);

					EditCOMPort.Enabled = true;
				}
			}
		}

		private void ButtonGetData_Click(object sender, System.EventArgs e)
		{
			int iError = 0;
			int liVal1 = 0;
			int liVal2 = 0;


			// Set to synchronous mode
			SQConfig(MODE_STOP, 0, 75, ref liVal1, ref liVal2);
			SQConfig(MODE_SYNC, 0, 75, ref liVal1, ref liVal2);


			// Get one packet of data
			iError = GetData(0);


			if(iError != 0)
			{
				DisplayErrorText(iError);
			}
		}

		private void ButtonStartPacketLoop_Click(object sender, System.EventArgs e)
		{
			cbDataSyncMode.Enabled = false;
			EditLoopTimer.Enabled = false;


			DataModeSwitch();


			// Run this procedure with an appropriate loop time.
			m_iLoopTime = Convert.ToInt32(EditLoopTimer.Text);
			Timer1.Interval = m_iLoopTime;

			// Enable timer.
			Timer1.Enabled = true;
		}

		private void ButtonEndPacketLoop_Click(object sender, System.EventArgs e)
		{
			int liVal1 = 0;
			int liVal2 = 0;


			SQConfig(MODE_STOP, 0, 75, ref liVal1, ref liVal2);


			Timer1.Enabled = false;


			cbDataSyncMode.Enabled = true;
			EditLoopTimer.Enabled = true;
		}

		private void Timer1_Tick(object Sender, EventArgs e)   
		{
			int iError = 0;


			// Get Inclinometer Data
			if(m_iGetDataBusy == 0)
			{
				m_iGetDataBusy = 1;


				iError = GetData(0);


				if(iError != 0)
				{
					DisplayErrorText(iError);
				}


				m_iGetDataBusy = 0;
			}
		}

		private void ButtonClearLatchedError_Click(object sender, System.EventArgs e)
		{
			int iComPort = Convert.ToInt32(EditCOMPort.Value);


			SetStatusText("Clearing Latched Error");

			
			SQClearLatchedError(iComPort);


			m_iLatchedError = 0;
			EditLatchedError.Text = Convert.ToString(m_iLatchedError);
		}

		private void ButtonSendReset_Click(object sender, System.EventArgs e)
		{
			int iComPort = Convert.ToInt32(EditCOMPort.Value);
			int iErrorCode = 0;


			SetStatusText("Resetting Device");


			iErrorCode = SQResetDevice(iComPort);


			if(iErrorCode == 0)
			{
				SetStatusText("Reset Successfull");
			}
			else
			{
				DisplayErrorText(iErrorCode);
			}


			m_iPacketCount = 0;
			EditPacketCount.Text = Convert.ToString(m_iPacketCount);

			m_iDroppedPackets = 0;
			EditDroppedPackets.Text = Convert.ToString(m_iDroppedPackets);

			m_iDuplicatePackets = 0;
			EditDuplicatePackets.Text = Convert.ToString(m_iDuplicatePackets);
		}

		private void ButtonFindFirstDevice_Click(object sender, System.EventArgs e)
		{
			// Open COM Port
			int iComStart = 1;
			int iComEnd = 20;


			SetStatusText("Starting Find First Device");
			SetStatusText("This may take some time ...");


			int iResult = SQFindFirstDevice(
				iComStart,
				iComEnd,
				115200,
				"N",
				8,
				1,
				0,
				0,
				0);

			if(iResult > 0)
			{
				string szMessage = "Found Device:  COM";
				SetStatusText(szMessage + Convert.ToString(iResult));
				EditCOMPort.Value = iResult;
			}
			else if(iResult == 0)
			{
				SetStatusText("Device Not Found");
			}
			else
			{
				DisplayErrorText(iResult);
			}


			SetStatusText("Ending Find First Device");
		}

		private void ButtonDone_Click(object sender, System.EventArgs e)
		{
			// Exit application
			Application.Exit();
		}

		private void ButtonLinearCodeTest_Click(object sender, System.EventArgs e)
		{
			int i = 0;
			int iResult = 0;
			int iCOMPort = 1;
			int iRetVal1 = 0;
			int iRetVal2 = 0;

			if(m_iCOMPortIsOpen == 0)
			{
				SetStatusText("Starting Linear Code Test");


				// Initialize DLL
				SQConfig(MODE_INIT_DLL, 0, 75, ref iRetVal1, ref iRetVal2);


				SetStatusText("Starting Find First Device");
				SetStatusText("This may take some time ...");

	
				// Find device (look in range COM1 to COM20)
				iResult = SQFindFirstDevice(
					1,
					20,
					115200,
					"N",
					8,
					1,
					0,
					0,
					0);


				SetStatusText("Ending Find First Device");


				// If you want to skip call to first device comment it out
				// and use the following line instead (ex. with COM12):
				//iResult = 12;

				// If port found
				if(iResult > 0)
				{
					string szMessage = "Found Device:  COM";
					SetStatusText(szMessage + Convert.ToString(iResult));


					// Store found port
					iCOMPort = iResult;

					// Open found port
					iResult = SQOpenPort(
						iCOMPort,
						115200,
						"N",
						8,
						1,
						0,
						0,
						0);

					// If open was successfull
					if(iResult == 0)
					{
						// Set to synchronous mode
						SQConfig(MODE_SYNC, 0, 75, ref iRetVal1, ref iRetVal2);


						// Attempt to read data
						int iMax = Convert.ToInt32(EditDataPoints.Text);
						for(i=0; i<iMax-10; i++)
						{
							GetData(iCOMPort);

							Update();
						}


						SetStatusText("Resetting Device");
						iResult = SQResetDevice(iCOMPort);


						for(i=0; i<10; i++)
						{
							GetData(iCOMPort);

							Update();
						}


						// Close port
						SQClosePort(iCOMPort);
					}
				}


				SetStatusText("Ending Linear Code Test");
			}
			else
			{
				SetStatusText("Must first close COM port");
			}
		}
	}
}
